/******************************************************************************/
/*Files to Include                                                            */
/******************************************************************************/

#include <htc.h>            /* HiTech General Includes */
#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */
#include "user.h"
#include "OLIMEX.h"

static unsigned char fault = 0;
static unsigned char j = 0;
static unsigned char i = 0;

/******************************************************************************/
/* Interrupt Routines                                                         */
/******************************************************************************/
void interrupt
isr(void)
{
	if(SSP1IF)
	{
		unsigned char status;
		unsigned char foo;
		int i;
		

/*-------------------------ROUTINES IN I2C SLAVE MODE-------------------------*/
		if(SSPCON1bits.SSPM == 0xE)
		{
			if(SSPSTAT & STOP_C)
			{

				//CHECK IF ADDRESS MATCH
				if(DATA_BUFFER[0] == MOD_RGB && DATA_BUFFER[1] == ADDRESS)
				{

					switch(DATA_BUFFER[2])
					{
					case START_PWM:
						StartPWM();
						break;
					case STOP_PWM:	
						StopPWM();
						break;
					case SET_RGB:
						SetPWM(	DATA_BUFFER[3],
							DATA_BUFFER[4],
							DATA_BUFFER[5]);
						break;
					case AUD_OFF:
						AUDIO_FLAG = 0;
						StopPWM();
						break;

					case AUD_ON:
						AUDIO_FLAG = 1;
						StartPWM();
						break;

					case SET_ADDR:
						SetAddress(DATA_BUFFER[3]);
						break;
					default:
						break;

					}
				}
			}
			else
			{
				status = SSPSTAT & 0x2D;	//DA | P | S | R/W | x | BF
				switch(status)
				{
				//STATE 1
				case STATE1:
				{
					foo = SSPBUF;
					DATA_INDEX = 0;
					for(i = 0; i < MAX_BUFFER_SIZE; i++)
						DATA_BUFFER[i] = 0;
					if (SEN)
						CKP = 1;
				}
					break;
				//STATE 2
				case STATE2:
				{
					DATA_BUFFER[DATA_INDEX++] = SSPBUF;
					if(DATA_INDEX >= MAX_BUFFER_SIZE)
						DATA_INDEX = 0;
					if (SEN)
						CKP = 1;
				}
					break;
				//STATE 3
				case STATE3:
				{
					foo = SSPBUF;
					while(BF);
				SSPBUF = 0x12;
					if (SEN)
						CKP = 1;
				}
					break;
				//STATE 4
				case STATE4:
				{
					foo = SSPBUF;
					while(BF);
					SSPBUF = 0x34;
					if (SEN)
						CKP = 1;
				}
					break;
				//STATE 5
				case STATE5:
				{

				}
					break;
				default:
					break;
				}
			}

		}
		SSP1IF = 0;

	}
	

	else if(PIR1bits.ADIF)
	{
		unsigned int result = 0;
		

		PIR1bits.ADIF = 0;

		
		if(ADCON0bits.CHS == TEMP_IND)
		{

			float vref;
			result = ADRESH;
			result <<= 8;
			result |= ADRESL;
			result = result - TEMP_OFFSET;		//CALIBRATION
		//	temp = (0.659 - (3.3/2.0)*(1 - 650/1023.0))/0.00132 - 40.0;	//GET CURRENT TEMP
			TEMPERATURE = -790 + 1.22*result;
		//	TEMPERATURE = (signed int)temp;
			
			vref  = V_OPEN_DRAN + (TEMP_COEF)*(TEMPERATURE - 0x17);		//CALCULATE VREF
			REFERENCE = (unsigned int)(vref*(310));

			j = 0;
			
			if(AUDIO_FLAG)
				ADCON0bits.CHS = AUD_IN;
			else
				ADCON0bits.CHS = SENSE_R;
				
		}
		else if(ADCON0bits.CHS == AUD_IN)
		{
			result = ADRESH;
			result <<= 8;
			result |= ADRESL;
			if(result > 574)
			{
				SetPWM(_R,_G,_B);
			}
		

			ADCON0bits.CHS = SENSE_R;
			

			
		}
		else
		{
			result = ADRESH;
			result <<= 8;
			result |= ADRESL;

		
			if(result > REFERENCE && result < 0x300)
			{
				if((result - REFERENCE) > V_FAULT)
					if(fault == 3)
					{
						StopPWM();
					}
					else
						fault++;
			}
			else
				fault = 0;

			if(!fault)
			{
				
				if(ADCON0bits.CHS == SENSE_R)
					ADCON0bits.CHS = SENSE_G;
				else if(ADCON0bits.CHS == SENSE_G)
					ADCON0bits.CHS = SENSE_B;
				else if(ADCON0bits.CHS == SENSE_B)
				{					
					if(AUDIO_FLAG)
						ADCON0bits.CHS = AUD_IN;
					else
						ADCON0bits.CHS = SENSE_R;					
				}
				if(!(++j))
					ADCON0bits.CHS = TEMP_IND;
			}
		}
		if(!DMX_FLAG)
			ADCON0bits.GO_nDONE = 1;
	

	}

	else if(PIR1bits.TMR1IF)
	{
		

		/*CHECK IF NO RISING EDGE IS DETECTED*/
		if(!PORT_STATE)
			BREAK = 1;

		/*SET TIMER1 FINISH FLAG*/
		T1CONbits.TMR1ON = 0;
		TIMER1_STATE = 1;

		/*CLEAR FLAG*/
		PIR1bits.TMR1IF = 0;

	}
	else if(INTCONbits.IOCIF)
	{
		/*CLEAR FLAG*/
	
		IOCAFbits.IOCAF5 = 0;

		/*DISABLE INTERUPTS*/
		IOCAPbits.IOCAP5 = 0;

		/*SET PORT FLAG*/
		PORT_STATE = 1;

		INTCONbits.IOCIF = 0;

	}
}