/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#include <htc.h>            /* HiTech General Includes */
#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */

#include "user.h"
#include "OLIMEX.h"


/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

void InitApp(void){
	InitGPIO();
	InitTemperatureSensor();
	InitAddress();
	InitTimer();
	InitADC();
	InitPWM();
	InitI2C();
	InitInterupts();

}

void InitAddress(void){
	unsigned int addr;
	addr = ReadFlash(MEMORY_ADDRESS);
	if((addr & 0xFF) == 0xFF)
		ADDRESS = DEFAULT_ADDRESS;
	else
		ADDRESS = addr & 0xFF;
}
void InitTemperatureSensor(void){
	FVRCON |= 0x20;
}
void InitGPIO(void){
	ANSELA = 0;
	ANSELC = 0;
	OPTION_REGbits.nWPUEN = 0;
	SDA_TRIS = 1;
	SCL_TRIS = 1;
	SDA_LAT = 0;
	SCL_LAT = 0;
	DMX_EN_TRIS = 1;		//PROGRAM PIN
	DMX_EN_LAT = 0;

	DMX_TRIS = 1;
	DMX_LAT = 0;
	WPUAbits.WPUA = 0;
	WPUAbits.WPUA5 = 1;
	

/*-----------------------------INIT ANALOG GPIOs------------------------------*/
	TRISAbits.TRISA0 = 1;	// ---> SENSE_R
	ANSELAbits.ANSA0 = 1;
	TRISAbits.TRISA1 = 1;	// ---> AUDIO_IN
	ANSELAbits.ANSA1 = 1;
	TRISCbits.TRISC2 = 1;	// ---> SENSE_B
	ANSELCbits.ANSC2 = 1;
	TRISAbits.TRISA4 = 1;	// ---> SENSE_G
	ANSELAbits.ANSA4 = 1;	
/*----------------------------------------------------------------------------*/


}
void InitInterupts(void){
	PIR1bits.ADIF = 0;
	PIR1bits.SSP1IF = 0;
	INTCONbits.TMR0IF = 0;
	PIR1bits.TMR1IF = 0;
	INTCONbits.IOCIF = 0;

	INTCONbits.IOCIE = 1;
	PIE1bits.ADIE = 1;
	PIE1bits.TMR1IE = 1;

	/*IF I2C IS IN MASTER MODE, DISABLE ISR*/
	if(SSPCON1 == 0x28)
		PIE1bits.SSP1IE = 0;
	else
		PIE1bits.SSP1IE = 1;
	
	INTCONbits.PEIE = 1;
	INTCONbits.GIE = 1;
	
}
void InitADC(void){
	ADCON1bits.ADCS = 0x05;		//Fosc/16 = 1MHz
	ADCON1bits.ADPREF = 0;
	ADCON1bits.ADFM = 1;		//Right justify
	ADCON0bits.CHS = TEMP_IND;
	ADCON0bits.ADON = 1;
	
	
	
}
void InitI2C(void){
	if(!DMX_EN_PORT)
	{
		/*I2C MASTER*/
		SSPCON2 = 0x00;
		SSPCON3 = 0x00;
		SSPSTAT = 0x00;
		SSPADD = 0x4E;			//50kHz
		SSPCON1 = 0x28;
	}
	else
	{
		/*I2C SLAVE*/
		SSPCON2 = 0x01;
		SSPCON3 = 0x40;
		SSPSTAT = 0x00;
		SSPADD = 0x48 << 1;
		SSPCON1 = 0x3E;
	}
}
void InitTimer(void){
	TMR1H = 0x00;
	TMR1L = 0x00;
	T1CONbits.T1CKPS = 0x00;
	T1CONbits.TMR1CS = 0x00;
	T1CONbits.nT1SYNC = 1;
	T1GCONbits.TMR1GE = 0;
	T1CONbits.TMR1ON = 0;

	OPTION_REGbits.TMR0CS = 0;
	OPTION_REGbits.PSA = 0;
	OPTION_REGbits.PS = 0x01;
}
void InitPWM(void){
	//DISABLE PWM OUTPUTS
	TRISCbits.TRISC5 = 1;
	TRISCbits.TRISC3 = 1;
	TRISAbits.TRISA2 = 1;

	//CLEAR PWMxCON
	PWM1CON = 0x00;
	PWM2CON = 0x00;
	PWM3CON = 0x00;

	//LOAD PWM PERIOD
	PR2 = 0xFF;		//~1kHz

	//CLEAR DUTY-CYCLE
	PWM1DCH = 0x00;
	PWM1DCL &= 0x3F;
	PWM2DCH = 0x00;
	PWM2DCL &= 0x3F;
	PWM3DCH = 0x00;
	PWM3DCL &= 0x3F;

	//CONFIGURE AND START TIMER 2
	PIR1bits.TMR2IF = 0;
	T2CONbits.T2CKPS = 0b11;
	T2CONbits.TMR2ON = 1;

	//ENABLE PWM OUTPUT AND WAIT FOR OVERFLOW
	while(!PIR1bits.TMR2IF);
	PIR1bits.TMR2IF = 0;
	
	//ENABLE PWM OUTPUTS
	TRISCbits.TRISC5 = 0;
	TRISCbits.TRISC3 = 0;
	TRISAbits.TRISA2 = 0;
	PWM1CON |= 0x40;
	PWM2CON |= 0x40;
	PWM3CON |= 0x40;
}

void StartPWM(void){	
	PWM1CON |= 0x80; 
	PWM2CON |= 0x80; 
	PWM3CON |= 0x80;
	ADCON0bits.ADON = 1;
	ADCON0bits.GO_nDONE = 1;	
}
void StopPWM(void){
	ADCON0bits.ADON = 0;
	PWM1CON &= 0x7F;
	PWM2CON &= 0x7F;
	PWM3CON &= 0x7F;
}

void SetPWM(unsigned char R, unsigned char G, unsigned char B){
	PWM1DCL = 0;
	PWM1DCH = R;
	PWM2DCL = 0;
	PWM2DCH = G;
	PWM3DCL = 0;
	PWM3DCH = B; 
	if(!AUDIO_FLAG)
	{
		_R = R;
		_G = G;
		_B = B;
	}
}

int ReadFlash(int address){
	int data = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.CFGS = 0;
	PMCON1bits.RD = 1;
	Nop();
	Nop();

	data = PMDATH;
	data <<= 8;
	data |= PMDATL;
	return data;
}
void EraseFlash(int address){
//	GIE = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.CFGS = 0;
	PMCON1bits.FREE = 1;
	PMCON1bits.WREN = 1;
	UnlockFlash();
	PMCON1bits.WREN = 0;
//	GIE = 1;
}
void UnlockFlash(void){
	PMCON2 = 0x55;
	PMCON2 = 0xAA;
	PMCON1bits.WR = 1;
	Nop();
	Nop();
}
void WriteFlash(int data, int address){
//	GIE = 0;
	PMCON1bits.CFGS = 0;
	PMADRL = address & 0x00FF;
	address >>= 8;
	PMADRH = address & 0x00FF;
	PMCON1bits.FREE = 0;
	PMCON1bits.LWLO = 1;
	PMCON1bits.WREN = 1;
	PMDATL = data & 0x00FF;
	data >>= 8;
	PMDATH = data & 0x00FF;
	PMCON1bits.LWLO = 0;
	UnlockFlash();
	PMCON1bits.WREN = 0;
//	GIE = 1;
}


void SetAddress(unsigned char addr){
	AUDIO_FLAG = 0;
	if(!DMX_EN_PORT)
	{		
		EraseFlash(MEMORY_ADDRESS);
		WriteFlash(addr, MEMORY_ADDRESS);
		ADDRESS = addr;
	}
}
void Fade(void){
	unsigned char i;
	unsigned int j;

	for(i = 0; i < 6; i++)
	{
		for(j = 0; j < 256; j++)
		{
			switch(i)
			{
			case 0:
				_R = j;
				_G = 0;
				_B = 255;
				break;
			case 1:
				_R = 255;
				_G = 0;
				_B = 255-j;
				break;
			case 2:
				_R = 255;
				_G = j;
				_B = 0;
				break;
			case 3:
				_R = 255-j;
				_G = 255;
				_B = 0;
				break;
			case 4:
				_R = 0;
				_G = 255;
				_B = j;
				break;
			case 5:
				_R = 0;
				_G = 255-j;
				_B = 255;
				break;
			}
			Delay(100);
		}
	}
}
void Delay(unsigned char delay){
	while(delay--)
	{
		//APROX 1us
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
	}
}
void GetTemperature(void){
	ADCON0bits.ADON = 0;
	ADCON0bits.CHS = TEMP_IND;
	ADCON0bits.ADON = 1;
	ADCON0bits.GO_nDONE = 1;

}


/*I2C MASTER MODE ROUTINES*/
void WaitSSP(void){
	while(!SSP1IF);
	SSP1IF = 0;

}
void StartI2C(void){
	SSPCON2bits.SEN = 1;
	WaitSSP();
}
void StopI2C(void){
	SSPCON2bits.PEN = 1;
	WaitSSP();
}
char I2CSendByte(unsigned char data){
    SSPBUF = data;
    WaitSSP();
    return(!SSPCON2bits.ACKSTAT);
}

unsigned char JumperIsOpen(void)
{
	/*
	 * If jumper is open -> return True
	 * If jumper is close -> return False
	 */
	while(1)
	{
		if(!DMX_EN_PORT)
		{
			Delay(20);
			if(!DMX_EN_PORT)
				return 0;
			else
				return 1;
		}
		else
			return 1;

	}
}

void StartDMX(void)
{
	unsigned char i = 0;
	unsigned int dmx_address = 0;
	unsigned char j;
	unsigned char RGB[3] = {0,0,0};
	unsigned int frame = 0;
	unsigned int data = 0;
	unsigned int new_address;

	DMX_FLAG = 1;
	PIE1bits.ADIE = 0;



	ADCON0bits.ADON = 0;
	StartPWM();
	TRISCbits.TRISC4 = 1;
	LATCbits.LATC4 = 0;

	T1CONbits.TMR1ON = 0;

	dmx_address = ReadFlash(DMX_ADDRESS);
	if(dmx_address == 0x3FFF)
		dmx_address = 0;

	while(1)
	{
		/* RESET DMX ADDRESS */
		while(JumperIsOpen())
		{
			SetPWM(128,128,128);
			dmx_address = 0;
			new_address = 0;
		}
		




	/*--------------------DETECT BREAK--------------------*/
		while(1)
		{
			PIR1bits.TMR1IF = 0;
			PIE1bits.TMR1IE = 1;
			GIE = 1;
			/*PACKAGE INITALIZATION*/
			BREAK = 0;
			PORT_STATE = 0;
			TIMER1_STATE = 0;
			TMR1H = 0xFF;
			TMR1L = 0x60;
			j = 3;
			frame = 1;


			/*DETECT FALLING EDGE*/
			INTCONbits.IOCIE = 0;
			IOCAPbits.IOCAP5 = 0;
			IOCANbits.IOCAN5 = 1;
			while(!IOCAFbits.IOCAF5);
			IOCAFbits.IOCAF5 = 0;

			/*START 40us TIMER*/
			T1CONbits.TMR1ON = 1;

			/*DISABLE FALLING EDGE DETECTION*/
			IOCANbits.IOCAN5 = 0;

			/*DETECT RISING EDGE*/
			INTCONbits.IOCIE = 1;
			IOCAPbits.IOCAP5 = 1;

			/*WAIT FOR TIMER TO OVERFLOW*/
			while(!TIMER1_STATE);
			PIE1bits.TMR1IE = 0;
			GIE = 0;
			if(BREAK)
				break;
		}
		

		
		INTCONbits.IOCIE = 0;
		IOCAPbits.IOCAP5 = 0;



/*--------------------READ START CODE--------------------*/
		/*DETECT START BIT*/


		IOCANbits.IOCAN5 = 1;
		while(!IOCAFbits.IOCAF5);
		IOCAFbits.IOCAF5 = 0;


		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");



		for(i = 8; i ; i--)
		{
			if(DMX_PORT)
				data |= 0x100;
			else
			{
				asm("NOP");
				asm("NOP");
				asm("NOP");
			}
			data >>= 1;
		}
/*------------------END READ START CODE--------------------*/
		if(data == DMX_START_CODE)
		{
			if(dmx_address != 0)
			{
				for(frame = 1; frame < 512; frame++)
				{
					if(frame == dmx_address)
					{
						/* Wait for falling edge*/
						while(DMX_PORT);

						/* 6us delay*/
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");

						/* Read 8 bits - LSB -> MSB*/

						for(i = 8; i ; i--)
						{
							/* Each cycle 4us */
							if(DMX_PORT)
								data |= 0x100;
							else
							{
								asm("NOP");
								asm("NOP");
								asm("NOP");
							}
							data >>= 1;
						}

						/* Store the data */
						RGB[0] = data;

						/* Same as above */
						while(DMX_PORT);

						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");

						for(i = 8; i ; i--)
						{
							if(DMX_PORT)
								data |= 0x100;
							else
							{
								asm("NOP");
								asm("NOP");
								asm("NOP");
							}
							data >>= 1;
						}

						RGB[1] = data;

						/* Same as above */

						while(DMX_PORT);

						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");

						for(i = 8; i ; i--)
						{
							if(DMX_PORT)
								data |= 0x100;
							else
							{
								asm("NOP");
								asm("NOP");
								asm("NOP");
							}
							data >>= 1;
						}

						RGB[2] = data;
						break;
					}
					else
					{
						while(DMX_PORT);

						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");
						asm("NOP");

						for(i = 8; i ; i--)
						{
							if(DMX_PORT)
								data |= 0x100;
							else
							{
								asm("NOP");
								asm("NOP");
								asm("NOP");
							}
							data >>= 1;
						}

					}
				}
			}
			else
			{
				for(frame = 1; frame < 512; frame++)
				{

					/* Wait for falling edge*/
					while(DMX_PORT);

					/* 6us delay*/
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");
					asm("NOP");

					/* Read 8 bits - LSB -> MSB*/

					for(i = 8; i ; i--)
					{
						/* Each cycle 4us */
						if(DMX_PORT)
							data |= 0x100;
						else
						{
							asm("NOP");
							asm("NOP");
							asm("NOP");
						}
						data >>= 1;

					}

					if(data != 0)
					{
						if(new_address == 0)
						{
							new_address = frame;
							new_address = 1;
						}
						else
						{
							if(frame == new_address)
							{
								dmx_address = frame;
								EraseFlash(DMX_ADDRESS);
								WriteFlash(dmx_address, DMX_ADDRESS);
							}
							break;
						}
					}
				}

			}


			/* Set RGB as the recieved data*/
			SetPWM(RGB[0], RGB[1], RGB[2]);			

		}
	}

}